/*******************************************************************************
* Copyright 2005-2022 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

/*
 *
 * fftw_execute_dft_r2c - FFTW3 wrapper to Intel(R) oneAPI Math Kernel Library (Intel(R) oneMKL).
 *
 ******************************************************************************
 */

#include "fftw3_mkl.h"
#ifdef DFT_ENABLE_OFFLOAD
#include "oneapi/mkl/export.hpp"
#include "fftw3_omp_offload_common.h"

DLL_EXPORT void
fftw_execute_dft_r2c_omp_offload(const fftw_plan plan, double *in, fftw_complex *out, void *interopObj)
{
    fftw_mkl_plan mkl_plan = (fftw_mkl_plan)plan;
    struct fftw_mkl_plan_s tmp_plan_s;

    if (!(mkl_plan && mkl_plan->execute_offload)) return;

    tmp_plan_s = mkl_plan[0];
    tmp_plan_s.io[0] = in;
    tmp_plan_s.io[1] = out;

    tmp_plan_s.execute_offload(&tmp_plan_s, interopObj);
}
#else
void
fftw_execute_dft_r2c(const fftw_plan plan, double *in, fftw_complex *out)
{
    fftw_mkl_plan mkl_plan = (fftw_mkl_plan)plan;
    struct fftw_mkl_plan_s tmp_plan_s;

    if (!(mkl_plan && mkl_plan->execute)) return;

    tmp_plan_s = mkl_plan[0];
    tmp_plan_s.io[0] = in;
    tmp_plan_s.io[1] = out;

    tmp_plan_s.execute(&tmp_plan_s);
}
#endif


